from uuid import uuid4

from django.contrib.auth import get_user_model
from django.test import TestCase, override_settings
from django.urls import reverse

from core.models.assemblies import Assembly
from core.models.conference import Conference, ConferenceMember
from core.models.rooms import Room


class MetricsTest(TestCase):
    def test_MetricsView_allow_ip_addresses(self):
        with self.settings(METRICS_SERVER_IPS=['1.2.3.4']):
            resp = self.client.get(reverse('metrics:index'))
            # Our address is not 1.2.3.4 so we should get a `HTTP 403 Forbidden` here.
            self.assertEqual(403, resp.status_code)

        with self.settings(METRICS_SERVER_IPS=['*']):
            resp = self.client.get(reverse('metrics:index'))
            # The settings will allow any IP address so we get HTTP 200.
            self.assertEqual(200, resp.status_code)

        with self.settings(METRICS_SERVER_IPS=['127.0.0.1']):
            resp = self.client.get(reverse('metrics:index'))
            # Our address is not 127.0.0.1 so we should get a `HTTP 200`.
            self.assertEqual(200, resp.status_code)

    @override_settings(METRICS_SERVER_IPS='*')
    def test_MetricsView(self):
        conf = Conference(slug='conf', name='TestConf', is_public=True)
        conf.save()
        assembly = Assembly(
            name='TestAssembly',
            slug='asmbly',
            state=Assembly.State('planned'),
            conference=conf,
        )
        assembly.save()
        room = Room(
            conference=conf,
            assembly=assembly,
            name='Room 1',
            room_type=Room.RoomType.BIGBLUEBUTTON,
            backend_status=Room.BackendStatus.ACTIVE,
            backend_link=str(uuid4()),
            backend_data={'close_secret': 'asdf'},
        )
        room.save()
        user = get_user_model()(username='testuser')
        user.save()

        member = ConferenceMember(conference=conf, active_angel=False, user=user)
        member.save()

        resp = self.client.get(reverse('metrics:index'))

        # Anzahl Konferenzen
        self.assertContains(resp, 'hub_conferences_total 1')

        # Anzahl User im System
        self.assertContains(resp, 'hub_users_total{user_type="human"} 1')

        # Anzahl Teilnehmer
        self.assertContains(resp, 'hub_conference_members{conference="conf",active_angel="false"} 1')
        self.assertContains(resp, 'hub_conference_members{conference="conf",active_angel="true"} 0')
        self.assertContains(resp, 'hub_conference_members_staff{conference="conf"} 0')

        # Anzahl eingelöste Tickets
        self.assertContains(resp, 'hub_conference_tickets{conference="conf"} 0')

        # Anzahl Assemblies: registriert, angenommen, abgelehnt
        self.assertContains(resp, 'hub_conference_assemblies{conference="conf",state="planned"} 1')

        # Anzahl Events
        self.assertContains(resp, 'hub_conference_events{conference="conf",kind="official"} 0')
        self.assertContains(resp, 'hub_conference_events{conference="conf",kind="assembly"} 0')
        self.assertContains(resp, 'hub_conference_events{conference="conf",kind="sos"} 0')

        # Anzahl Räume nach Typen: stage, bbb, ..
        self.assertContains(resp, 'hub_conference_rooms{conference="conf",room_type="bbb"} 1')
