import json
import uuid
import xml.etree.ElementTree as ET
from datetime import datetime, timedelta
from zoneinfo import ZoneInfo

from rest_framework.authtoken.models import Token

from django.test import TestCase, override_settings
from django.urls import reverse
from django.utils.timezone import now

from core.models import Assembly, Conference, ConferenceExportCache, Event, PlatformUser, Room

TEST_CONF_ID = uuid.uuid4()


@override_settings(SELECTED_CONFERENCE_ID=TEST_CONF_ID)
class ScheduleTest(TestCase):
    def setUp(self):
        tz = ZoneInfo('CET')
        self.conf = Conference(
            id=TEST_CONF_ID,
            slug='conf',
            name='TestConf',
            is_public=True,
            start=datetime(2020, 12, 27, 1, 23, 45, tzinfo=tz),
            end=datetime(2020, 12, 30, 23, 45, 00, tzinfo=tz),
        )
        self.conf.save()
        self.conf.tracks.create(slug='community', name='Community').save()
        self.conf.tracks.create(slug='security', name='Security').save()
        self.assembly = Assembly(name='TestAssembly', slug='asmbly', conference=self.conf, state=Assembly.State.PLACED)
        self.assembly.save()
        self.room = Room(conference=self.conf, assembly=self.assembly, name='Foo Room', room_type=Room.RoomType.STAGE)
        self.room.save()

        self.user = PlatformUser(username='bernd', is_active=True)
        self.user.save()

        self.token = Token(user=self.user)
        self.token.save()

    def _test_schedule_export(self, url_suffix: str, expected_content_type: str, content_parser):
        schedule_url = reverse('api:conference-schedule') + url_suffix
        resp = self.client.get(schedule_url)
        self.assertEqual(200, resp.status_code)
        self.assertEqual(resp['Content-Type'], expected_content_type)
        content_str = resp.content.decode('utf-8')

        # check if the (empty) result is parse able
        content_parser(content_str)

        # prepare assembly for event-hosting, incl. room
        self.assembly.state = Assembly.State.ACCEPTED
        self.assembly.save()
        r = self.assembly.rooms.create(conference=self.conf, name='Zelt', room_type='stage')

        # create an ordinary event

        ev1 = Event(
            conference=self.conf,
            assembly=self.assembly,
            room=r,
            name='Example Event',
            is_public=True,
            schedule_start=self.conf.start,
            schedule_duration=timedelta(minutes=42),
        )
        ev1.save()
        event_url = reverse('api:event-schedule', kwargs={'pk': ev1.pk})

        # check that event is in the output
        resp = self.client.get(schedule_url)
        self.assertEqual(200, resp.status_code)
        content_str = resp.content.decode('utf-8')
        content_parser(content_str)
        self.assertIn(ev1.name, content_str)

        if url_suffix == '.json':
            resp = self.client.get(event_url)
            self.assertEqual(200, resp.status_code)
            content_str = resp.content.decode('utf-8')
            content_parser(content_str)
            self.assertIn(ev1.name, content_str)

        # create too-short and non-public events
        ev2a = Event(
            conference=self.conf,
            assembly=self.assembly,
            room=r,
            name='Short Event',
            is_public=True,
            schedule_start=self.conf.start,
            schedule_duration=timedelta(minutes=2),
        )
        ev2a.save()
        ev2b = Event(
            conference=self.conf,
            assembly=self.assembly,
            room=r,
            name='Short Event',
            is_public=False,
            schedule_start=self.conf.start,
            schedule_duration=None,
        )
        ev2b.save()

        # check that none of the new events are in the output
        resp = self.client.get(schedule_url)
        self.assertEqual(200, resp.status_code)
        content_str = resp.content.decode('utf-8')
        content_parser(content_str)
        self.assertIn(ev1.name, content_str)
        self.assertNotIn(ev2a.name, content_str)
        self.assertNotIn(ev2b.name, content_str)

        # Add addintional data to event 1 to append the persons
        ev1.additional_data = {
            'id': 12334,
            'url': 'https://fahrplan.events.ccc.de/congress/2023/fahrplan/events/12334.html',
            'slug': 'Vogc-12334-eroffnung',
            'type': 'lecture',
            'track': 'CCC',
            'persons': [
                {
                    'id': 987654,
                    'guid': 'aaaaaaaa-0000-000d-d00d-1d1d1d1d1d1d',
                    'name': 'arthur',
                    'links': [],
                    'biography': 'Test speaker',
                    'public_name': 'arthur',
                }
            ],
        }
        ev1.save()

        # check that event is in the output
        resp = self.client.get(schedule_url)
        self.assertEqual(200, resp.status_code)
        content_str = resp.content.decode('utf-8')
        content_parser(content_str)
        self.assertIn(ev1.name, content_str)

        if url_suffix == '.json':
            resp = self.client.get(event_url)
            self.assertEqual(200, resp.status_code)
            content_str = resp.content.decode('utf-8')
            content_parser(content_str)
            self.assertIn(ev1.name, content_str)

    def test_export_json(self):
        self._test_schedule_export('.json', 'application/json', json.loads)

    def test_export_xml(self):
        self._test_schedule_export('.xml', 'text/xml', ET.fromstring)

    def testGetEvent(self):
        event = Event(conference=self.conf, assembly=self.assembly, name='Example Event', is_public=True)
        event.save()

        url = reverse('api:event-schedule', kwargs={'pk': event.pk})
        resp = self.client.get(url)
        self.assertEqual(200, resp.status_code, f'Unexpected result from GET: {resp.content}')

        data = json.loads(resp.content)
        self.assertEqual(str(event.pk), data['guid'])
        self.assertEqual(event.name, data['title'])

    def testPushExistingEvent(self):
        event = Event(conference=self.conf, assembly=self.assembly, name='Example Event', is_public=False)
        event.save()

        update = {
            'url': 'https://fahrplan.events.ccc.de/rc3/2020/Fahrplan/events/11583.html',
            'id': 11583,
            'guid': str(event.id),
            'logo': None,
            'date': '2020-12-27T12:20:00+01:00',
            'start': '12:20',
            'duration': '00:30',
            'room': 'foo room',
            'slug': 'rc3-11583-rc3_eroffnung',
            'title': '#rC3 Er\u00f6ffnung',
            'subtitle': '',
            'track': 'Community',
            'type': 'Talk',
            'language': 'de',
            'abstract': 'Willkommen zur ersten und hoffentlich einzigen Remote Chaos Experience!',
            'description': '',
            'recording_license': '',
            'do_not_record': False,
            'persons': [{'id': 14151, 'public_name': 'blubbel'}],
            'links': [],
            'attachments': [],
        }

        url = reverse('api:event-schedule', kwargs={'pk': event.pk})

        with self.modify_settings(API_USERS={'append': self.user.username}):
            resp = self.client.post(url, json.dumps(update), content_type='application/json', HTTP_AUTHORIZATION=f'Token {self.token.key}')

        self.assertEqual(201, resp.status_code, f'Unexpected result from POST: {resp.content}')

        event.refresh_from_db()
        self.assertFalse(event.is_public)
        self.assertTrue('rC3' in event.name, f'Expected "rC3" in event name "{event.name}".')
        self.assertEqual(timedelta(minutes=30), event.schedule_duration)
        self.assertIsNotNone(event.schedule_end)

        update = {'public': True}

        with self.modify_settings(API_USERS={'append': self.user.username}):
            resp = self.client.post(url, json.dumps(update), content_type='application/json', HTTP_AUTHORIZATION=f'Token {self.token.key}')

        self.assertEqual(201, resp.status_code, f'Unexpected result from POST: {resp.content}')

        event.refresh_from_db()
        self.assertTrue(event.is_public)
        self.assertTrue('rC3' in event.name, f'Expected "rC3" in event name "{event.name}".')
        self.assertEqual(timedelta(minutes=30), event.schedule_duration)
        self.assertIsNotNone(event.schedule_end)

    def testPushNewEvent(self):
        another_room = Room(conference=self.conf, assembly=self.assembly, room_type=Room.RoomType.STAGE)
        another_room.save()

        update = {
            'url': 'https://fahrplan.events.ccc.de/rc3/2020/Fahrplan/events/11583.html',
            'id': 11583,
            'guid': 'd9334deb-f183-4aec-9c6c-137741f6ff73',
            'logo': None,
            'date': '2020-12-27T12:20:00+01:00',
            'start': '12:20',
            'duration': '01:30',
            'room': 'foo room',
            'room_id': str(another_room.pk),
            'slug': 'rc3-11583-rc3_eroffnung',
            'title': '#rC3 Er\u00f6ffnung',
            'subtitle': '',
            'type': 'Talk',
            'language': 'de',
            'abstract': 'Willkommen zur ersten und hoffentlich einzigen Remote Chaos Experience!',
            'description': '',
            'recording_license': '',
            'do_not_record': False,
            'persons': [{'id': 14151, 'public_name': 'blubbel'}],
            'links': [],
            'attachments': [],
            'public': False,
        }

        self.assertFalse(Event.objects.filter(pk=update['guid']).exists())

        url = reverse('api:event-schedule', kwargs={'pk': update['guid']})

        with self.modify_settings(API_USERS={'append': self.user.username}), self.assertLogs('api.views.schedule', 'WARNING'):
            resp = self.client.post(url, json.dumps(update), content_type='application/json', HTTP_AUTHORIZATION=f'Token {self.token.key}')

        self.assertEqual(201, resp.status_code, f'Unexpected result from POST: {resp.content}')

        self.assertTrue(Event.objects.filter(pk=update['guid']).exists())
        event = Event.objects.get(pk=update['guid'])
        self.assertFalse(event.is_public)
        self.assertTrue('rC3' in event.name, f'Expected "rC3" in event name "{event.name}".')
        self.assertEqual(timedelta(minutes=90), event.schedule_duration)
        self.assertIsNotNone(event.schedule_end)
        self.assertIsNone(event.track)

        self.assertEqual(another_room.pk, event.room_id, 'Expected import to prefer "room_id" over "room".')

        update = {'public': False, 'track': 'Security'}

        with self.modify_settings(API_USERS={'append': self.user.username}):
            resp = self.client.post(url, json.dumps(update), content_type='application/json', HTTP_AUTHORIZATION=f'Token {self.token.key}')

        self.assertEqual(201, resp.status_code, f'Unexpected result from POST: {resp.content}')

        event.refresh_from_db()
        self.assertFalse(event.is_public)
        self.assertTrue('rC3' in event.name, f'Expected "rC3" in event name "{event.name}".')
        self.assertEqual(timedelta(minutes=90), event.schedule_duration)
        self.assertIsNotNone(event.schedule_end)
        self.assertIsNotNone(event.track)

    def testPushInvalidTrack(self):
        update = {
            'url': 'https://fahrplan.events.ccc.de/rc3/2020/Fahrplan/events/11583.html',
            'id': 11583,
            'guid': 'd9334deb-f183-4aec-9c6c-137741f6ff73',
            'logo': None,
            'date': '2020-12-27T12:20:00+01:00',
            'start': '12:20',
            'duration': '01:30',
            'room': 'foo room',
            'slug': 'rc3-11583-rc3_eroffnung',
            'title': '#rC3 Er\u00f6ffnung',
            'subtitle': '',
            'type': 'Talk',
            'language': 'de',
            'abstract': 'Willkommen zur ersten und hoffentlich einzigen Remote Chaos Experience!',
            'description': '',
            'recording_license': '',
            'do_not_record': False,
            'persons': [{'id': 14151, 'public_name': 'blubbel'}],
            'links': [],
            'attachments': [],
            'public': False,
            'track': 'Fnord',
        }

        self.assertFalse(Event.objects.filter(pk=update['guid']).exists())

        url = reverse('api:event-schedule', kwargs={'pk': update['guid']})

        with self.modify_settings(API_USERS={'append': self.user.username}), self.assertLogs('api.views.schedule', 'WARNING'):
            resp = self.client.post(url, json.dumps(update), content_type='application/json', HTTP_AUTHORIZATION=f'Token {self.token.key}')

        self.assertEqual(400, resp.status_code, f'Unexpected success result from POST: {resp.content}')

    def testScheduleExport(self):
        # TODO write this test. This just tests some duration formatting edge cases for now
        r = self.assembly.rooms.create(conference=self.conf, name='Zelt', room_type='stage')
        ev1 = Event(
            conference=self.conf,
            assembly=self.assembly,
            room=r,
            name='Example Event',
            is_public=True,
            schedule_start=self.conf.start,
            schedule_duration=timedelta(minutes=120),
        )
        ev1.save()

        resp = self.client.get(reverse('api:conference-schedule') + '.xml')
        self.assertEqual(resp.status_code, 200)
        content = ET.fromstring(resp.content.decode('utf-8'))
        day1, *remaining_days = content.findall('day')
        for remaining_day in remaining_days:
            self.assertEqual(len(remaining_day), 0)

        (room,) = day1
        (event1,) = room
        self.assertEqual(event1.findtext('title'), ev1.name)
        self.assertEqual(event1.findtext('date'), '2020-12-27T01:23:45+01:00')
        self.assertEqual(event1.findtext('start'), '01:23')
        self.assertEqual(event1.findtext('duration'), '02:00')

    def test_cache(self):
        event = Event(conference=self.conf, assembly=self.assembly, name='Example Event', is_public=False)
        event.save()

        # sanity check: we don't have a cached schedule
        self.assertFalse(ConferenceExportCache.objects.filter(type=ConferenceExportCache.Type.SCHEDULE).exists())

        # query schedule and check that an ETag got sent
        url = reverse('api:conference-schedule')
        resp = self.client.get(url)
        self.assertEqual(200, resp.status_code)
        self.assertIn('ETag', resp.headers)
        self.assertNotEqual(resp.headers['ETag'], '')

        # check the cached schedule
        self.assertEqual(1, ConferenceExportCache.objects.filter(type=ConferenceExportCache.Type.SCHEDULE).count())
        c = ConferenceExportCache.objects.filter(type=ConferenceExportCache.Type.SCHEDULE).first()
        self.assertEqual(self.conf, c.conference)
        self.assertEqual(resp.headers['ETag'], c.etag)

        # query schedule again but send ETag, should result in 'Not Modified' response
        resp2 = self.client.get(url, headers={'If-None-Match': resp.headers['ETag']})
        self.assertEqual(304, resp2.status_code)

        # update the event so the schedule should have changed -> we get a full response with new ETag
        event.last_update = now()
        event.save()
        resp3 = self.client.get(url, headers={'If-None-Match': resp.headers['ETag']})
        self.assertEqual(200, resp3.status_code)
        self.assertNotEqual(resp.headers['ETag'], resp3.headers['ETag'])

        # check the cached schedules again, should still be only one
        self.assertEqual(1, ConferenceExportCache.objects.filter(type=ConferenceExportCache.Type.SCHEDULE).count())
        c.refresh_from_db()
        self.assertEqual(self.conf, c.conference)
        self.assertEqual(resp3.headers['ETag'], c.etag)
