import logging

from django import forms
from django.contrib.gis.geos import Point
from django.core.exceptions import ValidationError

from core.models import (
    Assembly,
    MapPOI,
    PlatformUser,
    ScheduleSource,
)
from core.utils import str2timedelta

from backoffice.forms.assemblies import (
    AssemblyApplicationAddForm,
    AssemblyCreateForm,
    AssemblyEditForm,
    AssemblyMemberAddForm,
    AssemblyMemberEditForm,
)
from backoffice.forms.assemblies_team import (
    AssemblyTeamMessageForm,
)
from backoffice.forms.badges import (
    BadgeAssignForm,
    BadgeForm,
    BadgeTokenForm,
    BadgeTokenTimeConstraintForm,
    BadgeTokenTimeConstraintFormSet,
)
from backoffice.forms.events import EventForm
from backoffice.forms.rooms import (
    AssemblyRoomCreateBigBlueButtonForm,
    AssemblyRoomCreateForm,
    AssemblyRoomCreateGenericForm,
    AssemblyRoomCreateHangarForm,
    AssemblyRoomEditForm,
    AssemblyRoomEditHangarForm,
    AssemblyRoomLinkCreateForm,
)
from backoffice.forms.static_pages import (
    StaticPageCreateForm,
    StaticPageEditForm,
)

logger = logging.getLogger(__name__)


class ConferenceSelectionForm(forms.Form):
    conference_slug = forms.SlugField()


class ProfileForm(forms.ModelForm):
    class Meta:
        model = PlatformUser
        fields = [
            'show_name',
            # 'description',
            'status',
            'status_public',
            'timezone',
            'no_animations',
            'colorblind',
            'high_contrast',
            'receive_dms',
            'receive_dm_images',
            'autoaccept_contacts',
        ]


class ScheduleSourceCreateForm(forms.ModelForm):
    class Meta:
        model = ScheduleSource
        fields = ['import_url']

    assembly_slug = forms.CharField(max_length=100, required=True)
    import_type = forms.CharField(max_length=100, required=False)
    import_frequency = forms.CharField(max_length=30, required=True)

    def __init__(self, *args, **kwargs):
        self._conference = kwargs.pop('conference')
        super().__init__(*args, **kwargs)

    def clean_assembly_slug(self):
        assembly_slug = self.cleaned_data['assembly_slug']
        if assembly_slug == '*':
            return None
        if not Assembly.objects.filter(conference=self._conference, slug__iexact=assembly_slug).exists():
            raise ValidationError('An assembly with this slug does not exist.')
        return assembly_slug

    def clean(self):
        assembly_slug = self.cleaned_data.get('assembly_slug')
        self.instance.assembly = Assembly.objects.get(conference=self._conference, slug__iexact=assembly_slug) if assembly_slug else None
        self.instance.conference = self._conference
        try:
            self.instance.import_frequency = str2timedelta(self.cleaned_data.get('import_frequency'))
        except Exception:
            raise ValidationError('Invalid import frequency.')
        self.instance.import_type = self.cleaned_data['import_type']


class POIForm(forms.ModelForm):
    class Meta:
        model = MapPOI
        fields = ['visible', 'is_official', 'name_de', 'name_en', 'description_de', 'description_en', 'location_floor']

    location_point = forms.JSONField(required=False)

    def clean_location_point(self):
        if self.cleaned_data['location_point']:
            try:
                self.instance.location_point = self.get_location_point()
                return self.instance.location_point
            except AssertionError:
                raise ValidationError('Syntax Error')

    def get_location_point(self):
        if not (parsed := self.cleaned_data['location_point']):
            return None

        assert isinstance(parsed, list) and len(parsed) == 2

        pt = Point(parsed[0], parsed[1], srid=-1)
        return pt


__all__ = [
    'AssemblyApplicationAddForm',
    'AssemblyCreateForm',
    'AssemblyEditForm',
    'AssemblyMemberAddForm',
    'AssemblyMemberEditForm',
    'AssemblyRoomCreateBigBlueButtonForm',
    'AssemblyRoomCreateForm',
    'AssemblyRoomCreateGenericForm',
    'AssemblyRoomCreateHangarForm',
    'AssemblyRoomEditForm',
    'AssemblyRoomEditHangarForm',
    'AssemblyRoomLinkCreateForm',
    'AssemblyTeamMessageForm',
    'BadgeAssignForm',
    'BadgeForm',
    'BadgeTokenForm',
    'BadgeTokenTimeConstraintForm',
    'BadgeTokenTimeConstraintFormSet',
    'ConferenceSelectionForm',
    'EventForm',
    'POIForm',
    'ProfileForm',
    'ScheduleSourceCreateForm',
    'StaticPageCreateForm',
    'StaticPageEditForm',
]
