from typing import Any

from django.db.models.query import QuerySet
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from django.views.generic import DetailView, ListView
from rules.contrib.views import AutoPermissionRequiredMixin

from core.models import Assembly, Invitation

from backoffice.views.invitations.send import InvitationCreateView
from backoffice.views.invitations.update import InvitationDecisionView, InvitationUpdateView
from backoffice.views.mixins import ConferenceLoginRequiredMixin, ConferenceRuleRequiredMixin


class InvitationListView(ConferenceLoginRequiredMixin, ListView):
    model = Invitation
    template_name = 'backoffice/invitations/list.html'
    context_object_name = 'invitations'

    def get_queryset(self) -> QuerySet[Invitation]:
        return super().get_queryset().filter(state=Invitation.RequestsState.REQUESTED)

    def get_context_data(self, **kwargs):
        return {**super().get_context_data(**kwargs), 'active_page': 'invitations', 'title_text': _('Invitations')}


class InvitationDetailView(ConferenceRuleRequiredMixin, AutoPermissionRequiredMixin, DetailView):
    model = Invitation
    template_name = 'backoffice/invitations/detail.html'
    context_object_name = 'invitation'

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        if not Invitation.type_is(invitation := self.object):  # pragma: no cover
            raise ValueError('Invalid object type:')
        context = super().get_context_data(**kwargs)
        context['active_page'] = 'invitations'
        match invitation.type:
            case Invitation.InvitationType.HABITAT:
                context.update(
                    {
                        'title_text': _('Invitation__title__habitat'),
                        'introduction_text': _('Invitation__introduction__habitat'),
                        'requester_label': _('Assembly') if invitation.requester != Assembly.Hierarchy.REGULAR else _('Assembly__parent'),
                        'requester_link': reverse_lazy('backoffice:assembly', kwargs={'pk': invitation.requester.pk}),
                        'requested_label': _('Assembly') if invitation.requester == Assembly.Hierarchy.REGULAR else _('Assembly__parent'),
                        'requested_link': reverse_lazy('backoffice:assembly', kwargs={'pk': invitation.requested.pk}),
                    }
                )
            case Invitation.InvitationType.TEAM_TO_MEMBER:
                from core.templatetags.hub_absolute import hub_absolute

                context.update(
                    {
                        'title_text': _('Invitation__type__team_to_member'),
                        'introduction_text': _('Invitation__introduction__team'),
                        'requester_label': _('Team'),
                        'requested_label': _('TeamMember'),
                        'requester_link': reverse_lazy('backoffice:team', kwargs={'uuid': invitation.requester.uuid}),
                        'requested_link': hub_absolute('plainui:user', user_slug=invitation.requested.slug),
                    }
                )
            case Invitation.InvitationType.MEMBER_TO_TEAM:
                from core.templatetags.hub_absolute import hub_absolute

                context.update(
                    {
                        'title_text': _('Invitation__type__member_to_team'),
                        'introduction_text': _('Invitation__introduction__team'),
                        'requester_label': _('TeamMember'),
                        'requested_label': _('Team'),
                        'requester_link': hub_absolute('plainui:user', user_slug=invitation.requester.slug),
                        'requested_link': reverse_lazy('backoffice:team', kwargs={'uuid': invitation.requested.uuid}),
                    }
                )

            case _:  # pragma: no cover
                raise NotImplementedError

        return context


__all__ = [
    'InvitationCreateView',
    'InvitationDecisionView',
    'InvitationDetailView',
    'InvitationListView',
    'InvitationUpdateView',
]
