import logging
from typing import Any

from django.contrib import messages
from django.forms import modelformset_factory
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views.generic import FormView

from core.abuse import REPORT_CATEGORIES
from core.forms import LinkForm, LinkFormSet, ProjectForm
from core.models.activitylog import ActivityLogChange, ActivityLogEntry, model_form_valid_activitylog
from core.models.links import Link
from core.models.projects import Project
from core.views.list_views import FilteredListView

from backoffice.views.mixins import AssemblyMixin, ConferenceLoginRequiredMixin

logger = logging.getLogger(__name__)


class ProjectListViewMixin(ConferenceLoginRequiredMixin, FilteredListView):
    model = Project
    template_name = 'backoffice/project/list.html'
    context_object_name = 'projects'
    paginate_by = 'all'
    filters = {}

    def get_queryset(self):
        return Project.objects.manageable_by_user(self.conference, user=self.request.user)

    def get_context_data(self, *args, **kwargs):
        return {
            **super().get_context_data(*args, **kwargs),
            'user_list': True,
        }


class ProjectFormMixin(ConferenceLoginRequiredMixin, FormView):
    model = Project
    form_class = ProjectForm
    template_name = 'backoffice/project/create_edit.html'
    project_type = _('SelfOrganizedProject')

    def post(self, request: HttpRequest, *args: str, **kwargs: Any) -> HttpResponse:
        link_forms = self.get_links_formset(data=request.POST)
        form = self.get_form()
        if link_forms.is_valid() and form.is_valid():
            self.object = form.save()
            link_forms.set_linked_object(self.object)
            link_forms.save(self.object)
            return self.form_valid(form)
        else:
            return self.render_to_response(self.get_context_data(link_forms=link_forms, form=form))

    def get_links_formset(self, *args, **kwargs):
        return modelformset_factory(model=Link, form=LinkForm, formset=LinkFormSet, can_delete=True)(
            queryset=self.object.links.all() if self.object else Link.objects.none(), prefix='links', *args, **kwargs
        )

    def form_invalid(self, form):
        if form.publish:
            form.instance.is_public = False
            messages.error(self.request, _('Project__publish__failed %(project_type)s') % {'project_type': self.project_type})
        else:
            messages.error(self.request, _('Project__form__invalid %(project_type)s') % {'project_type': self.project_type})
        return super().form_invalid(form)

    @model_form_valid_activitylog(context='backoffice')
    def form_valid(self, form):
        logger.info(
            'changed %(project_type)s "%(project)s" %(assembly)sby %(owner)s%(user)s',
            {
                'project_type': self.project_type,
                'project': form.instance,
                'user': self.request.user,
                'owner': 'owner ' if form.instance.owner else '',
                'assembly': f' on {form.instance.assembly} ' if form.instance.assembly else '',
            },
        )

        if form.publish and form.instance.is_public:
            messages.success(
                self.request, _('Project__published %(project)s %(project_type)s') % {'project': form.instance.name, 'project_type': self.project_type}
            )
            form.instance.log_activity(
                user=self.request.user,
                kind=ActivityLogEntry.Kind.SYSTEM,
                comment='PUBLISHED',
                is_public=ActivityLogChange(new='yes', old='no'),
            )
        elif form.publish and not form.instance.is_public:
            messages.success(
                self.request, _('Project__recalled %(project)s %(project_type)s') % {'project': form.instance.name, 'project_type': self.project_type}
            )
            form.instance.log_activity(
                user=self.request.user,
                kind=ActivityLogEntry.Kind.SYSTEM,
                comment='RECALLED',
                is_public=ActivityLogChange(new='no', old='yes'),
            )
        elif form.create:
            messages.success(
                self.request, _('Project__created %(project)s %(project_type)s') % {'project': form.instance.name, 'project_type': self.project_type}
            )
        else:
            messages.success(
                self.request, _('Project__updated %(project)s %(project_type)s') % {'project': form.instance.name, 'project_type': self.project_type}
            )
        return HttpResponseRedirect(self.get_success_url())

    def get_form_kwargs(self, *args, **kwargs):
        publish = False
        if self.object:
            publish = (not self.object.is_public and self.request.GET.get('publish') == 'true') or (
                self.object.is_public and self.request.GET.get('recall') == 'true'
            )
        return {
            **super().get_form_kwargs(*args, **kwargs),
            'publish': publish,
            'conference': self.conference,
        }

    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        if 'form' not in kwargs and 'form' not in ctx:
            ctx['form'] = self.get_form()
        if 'link_forms' not in kwargs:
            ctx['link_forms'] = self.get_links_formset()
        return {
            **ctx,
            'project_type': self.project_type,
            'report_address': REPORT_CATEGORIES['content'][2],
        }


class AssemblyProjectFormMixin(AssemblyMixin, ProjectFormMixin):
    assembly_url_param = 'assembly'
    assembly_management = True
    project_type = _('AssemblyProject')

    def get_queryset(self):
        return Project.objects.manageable_by_user(self.conference, user=self.request.user).filter(assembly=self.assembly)

    def get_form_kwargs(self, *args, **kwargs):
        return {
            **super().get_form_kwargs(*args, **kwargs),
            'assembly': self.assembly,
        }

    def get_context_data(self, *args, **kwargs):
        return {
            **super().get_context_data(*args, **kwargs),
            'assembly': self.assembly,
            'project_type': self.project_type,
        }

    def get_success_url(self):
        return reverse('backoffice:assembly-project', kwargs={'assembly': self.assembly.id, 'pk': self.object.id})


class SoProjectFormMixin(ProjectFormMixin):
    project_type = _('SelfOrganizedProject')

    def get_queryset(self):
        return Project.objects.manageable_by_user(self.conference, user=self.request.user).filter(owner=self.request.user)

    def get_form_kwargs(self, *args, **kwargs):
        return {
            **super().get_form_kwargs(*args, **kwargs),
            'owner': self.request.user,
        }

    def get_context_data(self, *args, **kwargs):
        return {
            **super().get_context_data(*args, **kwargs),
            'project_type': self.project_type,
        }

    def get_success_url(self):
        return reverse('backoffice:so-project', kwargs={'pk': self.object.id})
