from typing import Any

from django.contrib.messages import ERROR, add_message
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect
from django.urls import reverse
from django.utils.translation import gettext as _
from django.views.generic import UpdateView
from django.views.generic.edit import DeleteView
from rules.contrib.views import AutoPermissionRequiredMixin

from core.models import ActivityLogChange, TeamMember
from core.models.teams.team_member import LastManagerError


class TeamMemberUpdateView(AutoPermissionRequiredMixin, UpdateView):
    model = TeamMember
    http_method_names = ['post']
    fields = ['can_manage']

    def form_valid(self, form):
        if not TeamMember.type_is(team_member := self.get_object()):  # pragma: no cover
            raise ValueError('The object is not a TeamMember.')
        team = team_member.team
        old_manages = ', '.join(team.members.filter(can_manage=True).values_list('user__display_name', flat=True))
        response = super().form_valid(form)
        team.log_activity(
            self.request.user,
            manages=ActivityLogChange(old=old_manages, new=', '.join(team.members.filter(can_manage=True).values_list('user__display_name', flat=True))),
        )
        return response

    def form_invalid(self, form):
        for error in form.errors['__all__']:
            add_message(self.request, ERROR, str(error))
        return HttpResponseRedirect(self.get_success_url())

    def get_success_url(self):
        return reverse('backoffice:team', kwargs={'uuid': self.kwargs['team']})


class TeamMemberDeleteView(AutoPermissionRequiredMixin, DeleteView):
    model = TeamMember

    template_name = 'backoffice/components/confirmation_modal.html'

    def post(self, request: HttpRequest, *args: str, **kwargs: Any) -> HttpResponse:
        """
        Delete the team member if the confirmation is confirmed.
        """
        if request.POST.get('confirmation', request.GET.get('confirmation', 'false')) != 'true':
            return self.get(request, *args, **kwargs)
        try:
            return super().post(request, *args, **kwargs)
        except LastManagerError:
            add_message(request, ERROR, _('TeamMember__delete__cannot_delete_last_manager'))
            return self.get(request, *args, **kwargs)

    def form_valid(self, form):
        if not TeamMember.type_is(team_member := self.get_object()):  # pragma: no cover
            raise ValueError('The object is not a TeamMember.')
        team = team_member.team
        old_members = ', '.join(team.members.all().values_list('user__display_name', flat=True))
        old_count = str(team.members.count())
        response = super().form_valid(form)
        team.log_activity(
            self.request.user,
            members=ActivityLogChange(old=old_members, new=', '.join(team.members.all().values_list('user__display_name', flat=True))),
            members_count=ActivityLogChange(old=old_count, new=str(team.members.count())),
        )

        return response

    def get_context_data(self, **kwargs) -> dict[str, Any]:
        if not TeamMember.type_is(team_member := self.get_object()):  # pragma: no cover
            raise ValueError('The object is not a TeamMember.')
        if team_member.user == self.request.user:
            return {
                'confirmation_title': _('TeamMember__leave__warning__header'),
                'confirmation_body': _('TeamMember__leave__warning__text %(team)s') % {'team': team_member.team.name},
                'confirmation_class': 'danger',
                'confirmation_submit': _('TeamMember__leave__submit'),
                'confirmation_cancel_url': reverse('backoffice:team', kwargs={'uuid': team_member.team.uuid}),
                **super().get_context_data(**kwargs),
            }
        return {
            'confirmation_title': _('TeamMember__delete__warning__header'),
            'confirmation_body': _('TeamMember__delete__warning__text %(team)s %(user)s') % {'team': team_member.team.name, 'user': team_member.user.username},
            'confirmation_class': 'danger',
            'confirmation_submit': _('TeamMember__delete__submit'),
            'confirmation_cancel_url': reverse('backoffice:team', kwargs={'uuid': team_member.team.uuid}),
            **super().get_context_data(**kwargs),
        }

    def get_success_url(self):
        return reverse('backoffice:team', kwargs={'uuid': self.kwargs['team']})
