import contextlib
import json
from base64 import b64encode

from django.conf import settings
from django.contrib.sites.shortcuts import get_current_site
from django.core.mail import EmailMessage
from django.template import loader
from django.urls import NoReverseMatch
from django.utils.translation import gettext_lazy as _

REPORT_CATEGORIES = {
    'abuse': (_('abuse_report_category-abuse'), 'Abuse', settings.ABUSE_RECIPIENTS.get('abuse', settings.ABUSE_DEFAULT_RECIPIENT)),
    'content': (_('abuse_report_category-content'), 'Content', settings.ABUSE_RECIPIENTS.get('content', settings.ABUSE_DEFAULT_RECIPIENT)),
    'person': (_('abuse_report_category-person'), 'Person', settings.ABUSE_RECIPIENTS.get('person', settings.ABUSE_DEFAULT_RECIPIENT)),
    'tech': (_('abuse_report_category-tech'), 'Technical', settings.ABUSE_RECIPIENTS.get('tech', settings.ABUSE_DEFAULT_RECIPIENT)),
    'unknown': (_('abuse_report_category-misc'), 'Unknown', settings.ABUSE_RECIPIENTS.get('unknown', settings.ABUSE_DEFAULT_RECIPIENT)),
}
EMAIL_TEMPLATE_NAME = 'core/report_mail_body.txt'


def report_content(request, reporter, category, reported_content, problem_message, proposed_solution, moderation_url):
    from core.templatetags.hub_absolute import hub_absolute  # pylint: disable=import-outside-toplevel

    current_site = get_current_site(request)
    site_name = current_site.name
    domain = current_site.domain

    assert category in REPORT_CATEGORIES, f'Unknown report category {category}'
    cat = REPORT_CATEGORIES[category]
    readable_category = cat[1]
    recipient = cat[2]

    context = {
        'domain': domain,
        'site_name': site_name,
        'reporter': reporter,
        'reported_content': reported_content,
        'category': category,
        'readable_category': readable_category,
        'message': problem_message,
        'message2': proposed_solution,
        'moderation_url': moderation_url,
    }

    with contextlib.suppress(NoReverseMatch):
        context['reporter_profile'] = hub_absolute('plainui:user_by_uuid', uuid=str(reporter.uuid))

    if settings.ABUSE_USE_REPORTER_AS_FROM:
        reporter_mail = reporter.get_primary_mail() if reporter and reporter.is_authenticated else []
    else:
        reporter_mail = None

    subject = f'New {readable_category} Report'
    body = loader.render_to_string(EMAIL_TEMPLATE_NAME, context)

    email = EmailMessage(
        subject=subject,
        body=body,
        from_email=f'"{reporter.display_name}" <{reporter_mail}>' if reporter_mail else None,
        to=[recipient],
        reply_to=[reporter_mail] if reporter_mail else None,
        headers={
            'Sender': settings.DEFAULT_FROM_EMAIL,
            'X-Hub-Version': b64encode(json.dumps(settings.APP_VERSION_INFO).encode('utf-8')).decode('utf-8'),
        },
    )
    email.send()
