from argparse import ArgumentTypeError, BooleanOptionalAction
from datetime import datetime
from zoneinfo import ZoneInfo

from rich.console import Console

from django.contrib.auth.models import Group
from django.core.management import call_command
from django.core.management.base import CommandError
from django.utils.text import slugify
from django_rich.management import RichCommand

from core.models.assemblies import Assembly
from core.models.badges import BadgeCategory
from core.models.conference import Conference
from core.models.pages import StaticPage, StaticPageNamespace

NS_INTRO = '_intro_'
INTRO_PAGES = {
    'assemblies': {
        'title': {
            'de': 'Was ist das hier?',
            'en': 'What is this place?',
        },
        'text': {
            'de': 'Hier findest du alle Assemblies, die dieses Jahr dabei sind.',
            'en': 'Here you can find all the assemblies taking part this year.',
        },
    },
    'assemblies_events': {
        'title': {
            'de': 'Assembly Vorträge',
            'en': 'Assembly Events',
        },
        'text': {
            'de': 'Schau dir an, was die Assemblies vorhaben!',
            'en': 'See what the assemblies are up to!',
        },
    },
    'badges': {
        'title': {
            'de': 'Was sind Badges?',
            'en': 'What are badges?',
        },
        'text': {
            'de': 'Hier findest du eine Liste der Badges, die von Assemblies erstellt wurden und die du während dieser Veranstaltung finden könntest.\r\nAssemblies können neue Badges im Backoffice erstellen!',  # noqa: E501
            'en': 'Here you can find a list of badges that assemblies have created and that you may find during this event.\r\nAssemblies can create new badges in the backoffice!',  # noqa: E501
        },
    },
    'projects': {
        'title': {
            'de': 'Was sind Projekte?',
            'en': 'What are projects?',
        },
        'text': {
            'de': 'Assemblies und auch Einzelpersonen stellen hier Projekte vor, an denen sie arbeiten. Schau dir die Liste an und füge vielleicht selbst eines hinzu?',  # noqa: E501
            'en': "Assemblies and individual people like to highlight projects they are working on. Here you'll find a list, consider adding your fancy project?",  # noqa: E501
        },
    },
    'reportcontent': {
        'title': {
            'de': 'DIES IST KEIN KONTAKTFORMULAR',
            'en': 'THIS IS NOT A CONTACT FORM',
        },
        'text': {
            'de': 'Diese Funktion ist dafür gedacht, bösartigen und/oder illegalen Inhalt zu melden. Sie ist nicht dafür gedacht, Änderungen im Wiki zu beantragen oder allgemeine Fragen zu beantworten. Dafür kontaktiere bitte die Orga. Danke!',  # noqa: E501
            'en': 'This function is intended to report malicious and/or illegal contents. It is not meant to request changes in wiki pages or ask questions. For that please contact the orga. Thanks!',  # noqa: E501
        },
    },
    'sos': {
        'title': {
            'de': 'Was ist das hier?',
            'en': 'What is this?',
        },
        'text': {
            'de': 'Hier kannst du selbst organisierte Vorträge und Workshops sehen. Vielleicht möchtest du selbst etwas anbieten?',  # noqa: E501
            'en': 'Here you can find sessions and workshops which are hosted by individuals. You might host your own?',
        },
    },
}


def seed_conference(conf: Conference, console: Console, year: int = 0):
    if Assembly.objects.filter(conference=conf).exists():
        raise ValueError('Conference already has assemblies?!')

    # create info block namespace and texts
    StaticPageNamespace.objects.create(conference=conf, prefix=NS_INTRO, groups=['hubteam'])
    for page_slug, page_data in INTRO_PAGES.items():
        for lang in ['de', 'en']:
            title = page_data['title'][lang]
            text = page_data['text'][lang]
            page = StaticPage.objects.create(
                conference=conf,
                slug=f'{NS_INTRO}{page_slug}',
                language=lang,
                title=title,
                protection=StaticPage.Protection.PERM,
                privacy=StaticPage.Privacy.NONE,
            )
            rev = page.revisions.create(body=text, title=title)
            rev.set_public()

    # create SoS assembly
    conf.self_organized_sessions_assembly = Assembly.objects.create(
        conference=conf,
        slug='sos',
        name='Self Organized Sessions',
        is_official=True,
        state=Assembly.State.HIDDEN,
    )

    # create navigation (top level)
    nav_conf = conf.nav_items.create(
        label_de='Konferenz',
        title_de='Offizielle Informationen, Tipps & Tricks zur Veranstaltung',
        label_en='Conference',
        title_en='official information, tips & tricks for the event',
        index=10,
    )
    nav_visitors = conf.nav_items.create(
        label_de='Besucher',
        title_de='Nützliche Links',
        label_en='visitors',
        title_en='helpful links',
        index=15,
    )
    nav_my = conf.nav_items.create(
        label_de='Mein Congress',
        title_de='personalisiertes Erlebnis',
        label_en='my congress',
        title_en='personalized experience',
        index=100,
    )

    entries = [
        # (parent, index, icon, label_de, label_en, title_de, title_en, url)
        (
            nav_conf,
            10,
            'rocket-takeoff',
            'Willkommen',
            'Welcome',
            'Startseite',
            'start page',
            'conference://index',
        ),
        (
            nav_conf,
            19,
            'calendar-event',
            'Events (Jetzt)',
            'events (now)',
            'das Vortragsprogramm als Liste',
            'talks, workshops and other events in list view',
            'conference://fahrplan?mode=list#now',
        ),
        (
            nav_conf,
            20,
            'calendar-week',
            'Fahrplan',
            'Schedule',
            'das Vortragsprogramm',
            'talks, workshops and other events',
            'conference://fahrplan',
        ),
        (
            nav_conf,
            21,
            'person-video3',
            'Self-organized Sessions',
            'Self-organized Sessions',
            'Workshops oder Diskussionsrunden in Assemblies oder den Workshopräumen',
            'workshops or discussions in Assemblies or workshop rooms',
            'conference://sos',
        ),
        (
            nav_conf,
            22,
            'lightning',
            'Lightning Talks',
            'lightning talks',
            'Deine fünf Minuten des Ruhms!',
            'Give a short introduction on a topic',
            'https://c3lt.de/',
        ),
        (
            nav_conf,
            50,
            'camera-reels',
            'Streams',
            'Streams',
            'Live-Streams der Vorträge',
            'live streaming of talks',
            'https://streaming.media.ccc.de/%SLUG%/',
        ),
        (
            nav_conf,
            110,
            'book',
            'Wiki',
            'Wiki',
            'gemeinsames Wissen',
            'shared knowledge',
            'wiki://start',
        ),
        (
            nav_conf,
            111,
            'book',
            'Letzte Änderungen (Wiki)',
            'Recent changes (Wiki)',
            'gemeinsames Wissen',
            'shared knowledge',
            'conference://static_page_global_history',
        ),
        (
            nav_conf,
            120,
            'person-arms-up',
            'Assemblies',
            'Assemblies',
            'Gruppen von Personen die dabei sind',
            'groups of people you can meet',
            'conference://assemblies',
        ),
        (
            nav_conf,
            300,
            'lightbulb',
            'Projekte',
            'Projects',
            None,
            None,
            'conference://projects',
        ),
        (
            nav_conf,
            400,
            'award',
            'Badges',
            'Badges',
            "Schnapp' sie dir alle!",
            'Gotta find them all!',
            'conference://badges',
        ),
        (
            nav_conf,
            9001,
            'wrench',
            'Maschinenraum',
            'Backoffice',
            'Assemblies, Events, etc. verwalten',
            'Manage Assemblies, Events, etc.',
            'conference://backoffice',
        ),
        (
            nav_my,
            10,
            'shield',
            'Meine Badges',
            'My Badges',
            'entdecke den Congress',
            'explore the congress',
            'conference://manage_badges',
        ),
        (
            nav_my,
            200,
            'stars',
            'Mein Fahrplan',
            'My Schedule',
            'favorisierte Vorträge',
            'your favorite events',
            'conference://fahrplan?mode=list&my=y&kind=all',
        ),
        (
            nav_visitors,
            10,
            'ticket-perforated',
            'Tickets',
            'Tickets',
            'Wie? Wann? Wo?',
            'how? when? where?',
            'https://events.ccc.de/congress/%YEAR%/infos/tickets.html',
        ),
        (
            nav_visitors,
            15,
            'buildings',
            'Gebäude',
            'Venue',
            'Wie benutzen wir das Gebäude',
            'How we use the venue',
            'wiki://venue',
        ),
        (
            nav_visitors,
            20,
            'clipboard-fill',
            'Bulletin Board',
            'Bulletin Board',
            'ein digitales schwarzes Brett: suche und finde Dinge',
            'a digital board: search & find stuff',
            'conference://board',
        ),
        (
            nav_visitors,
            30,
            'patch-question',
            'FAQ',
            'FAQ',
            'häufig gestellte Fragen',
            'frequently asked questions',
            'https://events.ccc.de/congress/%YEAR%/infos/fundamentals-and-faq.html',
        ),
    ]

    def customize(s):
        return (s or '').replace('%SLUG%', conf.slug).replace('%YEAR%', str(year))

    for parent, index, icon, label_de, label_en, title_de, title_en, url in entries:
        if not year and '%YEAR%' in url:
            continue
        parent.children.create(
            conference=conf,
            index=index,
            icon=icon,
            label_de=customize(label_de),
            label_en=customize(label_en),
            title_de=customize(title_de),
            title_en=customize(title_en),
            url=customize(url),
        )

    # create badge categories
    badge_category_general, _created = BadgeCategory.objects.get_or_create(
        slug='general',
        defaults={
            'name_de': 'Allgemein',
            'name_en': 'General',
            'symbol': 'award',
            'description_de': 'Kategorie für allgemeine Badges',
            'description_en': 'Category for general badges',
        },
    )
    badge_category_explore, _created = BadgeCategory.objects.get_or_create(
        slug='explore',
        defaults={
            'name_de': 'Erkunden',
            'name_en': 'Exploration',
            'symbol': 'search-heart',
            'description_de': 'Kategorie für Badges die mit dem Entdecken neuer Dinge zusammenhängen',
            'description_en': 'Category for badges related to discovering new things',
        },
    )
    badge_category_help, _created = BadgeCategory.objects.get_or_create(
        slug='help',
        defaults={
            'name_de': 'Hilfe',
            'name_en': 'Help',
            'symbol': 'person-raised-hand',
            'description_de': 'Kategorie für Badges die für geleistete Unterstützung vergeben werden',
            'description_en': 'Category for badges that are awarded for provided support',
        },
    )
    conf.badge_categories.add(badge_category_general, badge_category_explore, badge_category_help)


def _validate_date(s: str) -> datetime:
    try:
        return datetime.strptime(s, '%Y')  # noqa: DTZ007
    except ValueError as exc:
        raise ArgumentTypeError(f"Not a valid date: {s!r}. Use format 'YYYY'") from exc


class Command(RichCommand):
    def add_arguments(self, parser):
        parser.add_argument(
            'name',
            type=str,
            nargs='?',
            help='The name of the new conference',
        )
        parser.add_argument(
            '-s',
            '--slug',
            type=str,
            help='The url slug of the new conference. If not given, it will be slugified from the name.',
            default=None,
        )
        parser.add_argument(
            '-y',
            '--year',
            type=_validate_date,
            help=(
                'The year of the CCC congress. If not given, it will use the current year.'
                ' Setting this to 0 will skip any year-specific setup (e.g., nav entries).'
            ),
            default=None,
        )
        parser.add_argument(
            '--bootstrap',
            action=BooleanOptionalAction,
            help=("Initialize the conference permission groups (e.g. 'Assembly-Teams'). If not given, execution is determined by group presence in database."),
        )
        parser.add_argument('-i', '--interactive', action='store_true', help='Ask for missing arguments', default=False)

    def handle(self, *args, **options):
        interactive = options.get('interactive', False)
        bootstrap = options.get('bootstrap')
        if not (name := options.get('name')):
            if not interactive:
                self.print_help('manage.py', 'create_conference')
                raise CommandError('Missing required argument: name. Use --interactive to be asked for missing arguments.')
            name = input('NAME of the new conference (long title): ')
        if not (slug := options.get('slug')) and interactive:
            slug = input('SLUG of the new conference, e.g. "42c3": ')
        elif not (slug := options.get('slug')):
            slug = slugify(name)
        if not (year := options.get('year')) and interactive:
            year_valid = False
            while not year_valid:
                year = input("YEAR of the CCC congress (leave empty if it's something else): ")
                try:
                    year = datetime.strptime(year, '%Y')  # noqa: DTZ007
                    year_valid = True
                except ValueError:
                    self.stderr.write('Invalid year format. Please use YYYY.')
        elif not (year := options.get('year')):
            year = datetime.now()  # noqa: DTZ005
        year = year.year

        self.console.print(
            f'Creating conference [bold blue]{slug}[/bold blue] (name: [bold blue]{name}[/bold blue]){ f' for year {year}' if year != 0 else ""}'
        )

        # check if the conference already exists
        if Conference.objects.filter(slug=slug).exists():
            raise CommandError(f'Conference with slug "{slug}" already exists.')

        if bootstrap and Group.objects.exists():
            answer = None
            while answer not in ['y', 'n', '']:
                answer = input('Groups are already present in the database. Do you still want to bootstrap? (y/[n]) ')

            bootstrap = answer.lower() == 'y'
            if bootstrap:
                self.console.print('Groups will be loaded and may be duplicated!', style='yellow bold')
            else:
                self.console.print('Bootstrap will not be applied.', style='yellow bold')

        # load our bootstrap fixtures
        if bootstrap or (bootstrap is None and not Group.objects.exists()):
            with self.console.status('Bootstrapping groups..') as status:
                call_command('loaddata', 'bootstrap_auth_groups.json')
                self.console.print('Creating conference... [green]done[/green]', style='green bold')
        elif bootstrap is None and Group.objects.exists():
            self.console.print('Groups are already present in the database.', style='green bold')

        with self.console.status('Creating conference...') as status:
            # create the new conference
            conf = Conference.objects.create(
                slug=slug,
                name=name,
                is_public=True,
            )

            # for CCC Jahresendveranstaltung we can guess some timestamps from the current year alone
            if year > 1970:
                status.update('Setting deadline...')
                conf.start = datetime(year, 12, 27, 10, 0, 0, tzinfo=ZoneInfo('Europe/Berlin'))
                conf.end = datetime(year, 12, 30, 18, 0, 0, tzinfo=ZoneInfo('Europe/Berlin'))
                conf.registration_deadline = datetime(year, 12, 1, 0, 0, 0, tzinfo=ZoneInfo('Europe/Berlin'))
                self.console.print('Setting deadline... [green]done[/green]', style='bold')

            # save everything
            conf.save()
        self.console.print('Creating conference... [green]done[/green]', style='bold')

        # fill in initial data
        with self.console.status('Setting up initial data for [blue bold]{conf.slug}[/blue bold]...') as status:
            seed_conference(conf, console=self.console, year=year)
        self.console.print(f'Setting up initial data for [blue bold]{conf.slug}[/blue bold]... [green]done[/green]', style='bold')
