from uuid import uuid4

from django.conf import settings
from django.db import models
from django.template.loader import render_to_string
from django.urls import NoReverseMatch, reverse
from django.utils.text import format_lazy
from django.utils.translation import gettext_lazy as _

from core.fields import ConferenceReference
from core.markdown import render_markdown


class DirectMessage(models.Model):
    id = models.UUIDField(primary_key=True, editable=False, default=uuid4)

    conference = ConferenceReference(blank=True, null=True)
    """The context of this message, primarily used if sender=None."""

    sender = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        blank=True,
        null=True,
        on_delete=models.CASCADE,
        related_name='sent_messages',
        help_text=_('DirectMessage__sender__help'),
        verbose_name=_('DirectMessage__sender'),
    )
    """The sender of this message or None if it is a system message (in which case the conference field is being used)."""

    recipient = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name='received_messages',
        help_text=_('DirectMessage__recipient__help'),
        verbose_name=_('DirectMessage__recipient'),
    )
    """The intended recipient of this message."""

    timestamp = models.DateTimeField(auto_now_add=True, help_text=_('DirectMessage__timestamp__help'), verbose_name=_('DirectMessage__timestamp'))
    """Timestamp when the message was sent."""

    autodelete_after = models.DateTimeField(
        blank=True, null=True, help_text=_('DirectMessage__auto_delete_after__help'), verbose_name=_('DirectMessage__auto_delete_after')
    )
    """Automatic purging deletes this message after this timestamp."""

    in_reply_to = models.ForeignKey(
        'self', blank=True, null=True, on_delete=models.SET_NULL, help_text=_('DirectMessage__in_reply_to__help'), verbose_name=_('DirectMessage__in_reply_to')
    )
    """Original message to which this is a reply."""

    subject = models.CharField(max_length=200, help_text=_('DirectMessage__subject__help'), verbose_name=_('DirectMessage__subject'))
    """Subject of the message, might include 'Re:' markers."""

    body = models.TextField(help_text=_('DirectMessage__body__help'), verbose_name=_('DirectMessage__body'))
    """Message with (limited) Markdown support."""

    deleted_by_sender = models.BooleanField(
        default=False, help_text=_('DirectMessage__deleted_by_sender__help'), verbose_name=_('DirectMessage__deleted_by_sender')
    )

    deleted_by_recipient = models.BooleanField(
        default=False, help_text=_('DirectMessage__deleted_by_recipient__help'), verbose_name=_('DirectMessage__deleted_by_recipient')
    )

    was_read = models.BooleanField(default=False, help_text=_('DirectMessage__was_read__help'), verbose_name=_('DirectMessage__was_read'))
    """Signals whether the recipient has read the message, this is *not* shown to the sender!"""

    has_responded = models.BooleanField(default=False, help_text=_('DirectMessage__has_responded__help'), verbose_name=_('DirectMessage__has_responded'))
    """Signals whether the recipient has replied to the message. (speed optimization to circumvent EXISTS query)"""

    flagged_for_abuse = models.BooleanField(
        default=False, help_text=_('DirectMessage__flagged_for_abuse__help'), verbose_name=_('DirectMessage__flagged_for_abuse')
    )
    """Messages by abusive users will be hidden, at least if the messages have not been read yet."""

    @property
    def is_system_message(self):
        return self.sender_id is None

    def send_notification(self):
        if self.sender is not None:
            subj = format_lazy(_('DirectMessage__notification_subject__user'), sender=self.sender, subject=self.subject)
        else:
            subj = format_lazy(_('DirectMessage__notification_subject__system'), subject=self.subject)
        ctx = {
            'conference': self.conference,
            'subject': self.subject,
            'message': self.body,
            'timestamp': self.timestamp,
            'sender': str(self.sender) if self.sender is not None else None,
            'recipient': str(self.recipient) if self.recipient is not None else None,
        }

        body_text = render_to_string('core/directmessage.txt', ctx)

        if settings.SUPPORT_HTML_MAILS:
            ctx['message_html'] = render_markdown(self.conference, self.body)
            body_html = render_to_string('core/directmessage.html', ctx)
        else:
            body_html = None

        try:
            link = reverse('directmessage-from-notification', kwargs={'pk': self.id})
        except NoReverseMatch:
            # TODO: generic URL via settings?
            link = None

        self.recipient.notify_user(subject=subj, message_text=body_text, message_html=body_html, details_link=link)
