from pathlib import Path
from typing import TypeIs
from uuid import uuid4

from django.conf import settings
from django.contrib.contenttypes.fields import GenericRelation
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import QuerySet
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _

from core.fields import ConferenceReference
from core.markdown import compile_translated_markdown_fields, store_relationships
from core.models.activitylog import ActivityLogMixin
from core.models.assemblies import Assembly
from core.models.base_managers import ConferenceManagerMixin
from core.models.conference import ConferenceMember
from core.models.tags import TaggedItemMixin
from core.models.users import PlatformUser
from core.validators import FileSizeValidator, ImageDimensionValidator


class ProjectManager(ConferenceManagerMixin['Project']):
    assembly_filter = 'assembly'
    select_related_fields = ['assembly', 'owner']
    prefetch_related_fields = ['links', 'assembly__members']

    def apply_public_filter(self, queryset: QuerySet['Project']) -> QuerySet['Project']:
        return queryset.filter(is_public=True).exclude(blocked=True)

    def apply_assembly_rights_filter(self, queryset: QuerySet['Project'], member: ConferenceMember, *, only_manageable: bool | None) -> QuerySet['Project']:
        return super().apply_assembly_rights_filter(queryset, member, only_manageable=only_manageable)

    def apply_self_organized_rights_filter(
        self, queryset: QuerySet['Project'], member: ConferenceMember, *, only_manageable: bool | None = None
    ) -> QuerySet['Project']:
        return queryset.filter(owner=member.user)


def get_banner_file_name(instance: 'Assembly', filename: str) -> str:
    return str(Path('proj') / str(instance.id) / 'banner' / filename)


def get_banner_help_test() -> str:
    return _('Project__banner_image__help %(limit)d, %(max_width)d, %(max_height)d') % {
        'limit': settings.ASSEMBLY_BANNER_FILE_SIZE_LIMIT,
        'max_width': settings.ASSEMBLY_BANNER_WIDTH_MAXIMUM,
        'max_height': settings.ASSEMBLY_BANNER_HEIGHT_MAXIMUM,
    }


class Project(TaggedItemMixin, ActivityLogMixin, models.Model):
    class Meta:
        verbose_name = _('Project')
        verbose_name_plural = _('Projects')
        unique_together = [['assembly', 'owner', 'name']]

    objects = ProjectManager()

    id = models.UUIDField(default=uuid4, primary_key=True, editable=False)
    conference = ConferenceReference(related_name='projects')
    assembly = models.ForeignKey(
        Assembly,
        null=True,
        blank=True,
        related_name='projects',
        on_delete=models.CASCADE,
        help_text=_('Project__assembly__help'),
        verbose_name=_('Project__assembly'),
    )
    owner = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        blank=True,
        null=True,
        related_name='projects',
        on_delete=models.CASCADE,
        help_text=_('Project__owner__help'),
        verbose_name=_('Project__owner'),
    )

    is_public = models.BooleanField(default=False, help_text=_('Project__is_public__help'), verbose_name=_('Project__is_public'))
    blocked = models.BooleanField(default=False, help_text=_('Project__blocked__help'), verbose_name=_('Project__blocked'))

    name = models.CharField(max_length=200)
    """Name of the room as it can be found in the assembly."""

    slug = models.SlugField(unique=True)

    description = models.TextField(blank=True, default='', help_text=_('Project__description__help'), verbose_name=_('Project__description'))
    """Description of the room/project."""
    description_html = models.TextField(blank=True, null=True)
    location = models.CharField(max_length=200, blank=True, default='', help_text=_('Project__location__help'), verbose_name=_('Project__location'))

    last_update = models.DateTimeField(auto_now=True)
    links = GenericRelation('Link', object_id_field='object_id', content_type_field='content_type', related_query_name='project_links')
    tags = GenericRelation('TagItem', object_id_field='target_id', content_type_field='target_type', related_query_name='assemblies')
    favorite_of = models.ManyToManyField(
        PlatformUser, related_name='favorite_projects', help_text=_('Project__favorite_of__help'), verbose_name=_('Project__favorite_of')
    )

    banner_image_height = models.PositiveIntegerField(null=True)
    banner_image_width = models.PositiveIntegerField(null=True)
    banner_image = models.ImageField(
        blank=True,
        null=True,
        height_field='banner_image_height',
        width_field='banner_image_width',
        help_text=get_banner_help_test,
        upload_to=get_banner_file_name,
        verbose_name=_('Assembly__banner_image'),
        validators=[
            FileSizeValidator(settings.ASSEMBLY_BANNER_FILE_SIZE_LIMIT),
            ImageDimensionValidator(
                min_size=(
                    settings.ASSEMBLY_BANNER_WIDTH_MINIMUM,
                    settings.ASSEMBLY_BANNER_HEIGHT_MINIMUM,
                ),
                max_size=(
                    settings.ASSEMBLY_BANNER_WIDTH_MAXIMUM,
                    settings.ASSEMBLY_BANNER_HEIGHT_MAXIMUM,
                ),
            ),
        ],
    )

    def __str__(self):
        return self.name

    @classmethod
    def type_is(cls, obj: object) -> 'TypeIs[Project]':
        return isinstance(obj, cls)

    def get_absolute_url(self):
        from core.templatetags.hub_absolute import hub_absolute  # pylint: disable=import-outside-toplevel

        if self.assembly:
            return hub_absolute('plainui:project', slug=self.slug, i18n=settings.ARCHIVE_MODE)
        return hub_absolute('plainui:so_project', slug=self.slug, i18n=settings.ARCHIVE_MODE)

    def get_state_display(self):
        return 'blocked' if self.blocked else 'public' if self.is_public else 'private'

    @property
    def conference_slug(self):
        return self.conference.slug

    @property
    def assembly_slug(self):
        return self.assembly.slug if self.assembly is not None else None

    @property
    def is_self_organized(self):
        return self.assembly is None

    def __create_slug(self, extension='', max_length: int = 50):
        """
        recursive function to generate a free room slug based on the room name
        """
        sluged = slugify(self.name + extension)
        if max_length and len(sluged) > max_length:
            sluged = sluged[:max_length]

        exists = False
        if self.id:
            exists = Project.objects.exclude(id=self.id).filter(slug=sluged).exists()
        else:
            exists = Project.objects.filter(slug=sluged).exists()

        if exists:
            extension = str(uuid4())[13:18]
            return self.__create_slug(extension)
        else:
            return sluged

    def generate_slug(self):
        self.slug = self.__create_slug()

    def clean(self):
        if not self.owner and not self.assembly:
            raise ValidationError(_('Project__owner_or_assembly_required'))
        if self.owner and self.assembly:
            raise ValidationError(_('Project__owner_and_assembly_not_allowed'))

    def save(self, *args, update_fields=None, **kwargs):
        if update_fields is None or 'description' in update_fields:
            render_results = compile_translated_markdown_fields(self, self.conference, 'description')
            store_relationships(self.conference, self, render_results)

        if not self.slug:
            self.generate_slug()

        return super().save(*args, update_fields=update_fields, **kwargs)
