from datetime import timedelta

from django.test import TestCase
from django.utils.timezone import now

from core.models.assemblies import Assembly
from core.models.conference import Conference
from core.models.rooms import Room, RoomShare


class RoomTests(TestCase):
    def setUp(self):
        self.conference = Conference(slug='foo', name='Foo Conference', start=now() - timedelta(days=1), end=now() + timedelta(days=1))
        self.conference.save()

        self.assembly1 = Assembly(conference=self.conference, slug='fnord', name='Fnord Assembly', state=Assembly.State.ACCEPTED, is_official=True)
        self.assembly1.save()
        self.assembly2 = Assembly(conference=self.conference, slug='banana', name='Obstgarten', state=Assembly.State.ARRIVED_CONFIRMED)
        self.assembly2.save()
        self.assembly3 = Assembly(conference=self.conference, slug='third', name='3. Rad', state=Assembly.State.ACCEPTED)
        self.assembly3.save()
        self.assembly4 = Assembly(conference=self.conference, slug='gone', name='never shown', state=Assembly.State.HIDDEN)
        self.assembly4.save()

        self.room1a = Room(conference=self.conference, assembly=self.assembly1, room_type=Room.RoomType.LECTURE_HALL, name='1a')
        self.room1a.save()
        self.room1b = Room(conference=self.conference, assembly=self.assembly1, room_type=Room.RoomType.STAGE, name='1b')
        self.room1b.save()
        self.room2a = Room(conference=self.conference, assembly=self.assembly2, room_type=Room.RoomType.WORKSHOP, name='2a')
        self.room2a.save()
        self.room2b = Room(conference=self.conference, assembly=self.assembly2, room_type=Room.RoomType.OUTSIDE, name='2b')
        self.room2b.save()
        # assembly3 does not have a room, on purpose
        self.room4a = Room(conference=self.conference, assembly=self.assembly4, room_type=Room.RoomType.STAGE, name='4a')
        self.room4a.save()

    def test_assignable_basics(self):
        tf_start = now() - timedelta(hours=1)
        tf_end = now() + timedelta(hours=1)

        # sanity check: without context no room should be available
        actual_qs = Room.objects.assignable_in_timeframe(self.conference, start=tf_start, end=tf_end)
        self.assertSetEqual(set(), set(actual_qs.values_list('name', flat=True)))

        # initial situation: assembly1 can only use rooms of itself (no shares yet)
        actual_qs = Room.objects.assignable_in_timeframe(self.conference, assembly=self.assembly1, start=tf_start, end=tf_end)
        self.assertSetEqual({'1a', '1b'}, set(actual_qs.values_list('name', flat=True)))

        # share assembly2's first room with assembly1
        share2a = self.room2a.shares.create(single_assembly=self.assembly1, enabled=True, timeframe=None)

        # now assembly1 should see room 2a, too
        actual_qs = Room.objects.assignable_in_timeframe(self.conference, assembly=self.assembly1, start=tf_start, end=tf_end)
        self.assertSetEqual({'1a', '1b', '2a'}, set(actual_qs.values_list('name', flat=True)))

        # disable share and we should be back to before
        share2a.enabled = False
        share2a.save(update_fields=['enabled'])
        actual_qs = Room.objects.assignable_in_timeframe(self.conference, assembly=self.assembly1, start=tf_start, end=tf_end)
        self.assertSetEqual({'1a', '1b'}, set(actual_qs.values_list('name', flat=True)))

        # there should be no SOS rooms yet
        actual_qs = Room.objects.assignable_in_timeframe(self.conference, start=tf_start, end=tf_end, is_sos=True)
        self.assertSetEqual(set(), set(actual_qs.values_list('name', flat=True)))

        # share assembly2's second room for Self-Organized-Sessions
        self.room2b.shares.create(enabled=True, all_self_organized_sessions=True)

        # now that room should be picked up for SOS
        actual_qs = Room.objects.assignable_in_timeframe(self.conference, start=tf_start, end=tf_end, is_sos=True)
        self.assertSetEqual({'2b'}, set(actual_qs.values_list('name', flat=True)))

        # for members of that assembly, the assembly's rooms should show up for SOS, too
        actual_qs = Room.objects.assignable_in_timeframe(self.conference, assembly=self.assembly1, start=tf_start, end=tf_end, is_sos=True)
        self.assertSetEqual({'1a', '1b', '2b'}, set(actual_qs.values_list('name', flat=True)))

    def test_assignable_ranged(self):
        tf_middle = now()
        tf_start = tf_middle - timedelta(hours=1)
        tf_end = tf_middle + timedelta(hours=1)

        # sanity checks
        self.assertEqual(0, RoomShare.objects.count(), 'Expected no room shares at start of test.')
        self.assertEqual(
            0,
            Room.objects.assignable_in_timeframe(self.conference, assembly=self.assembly3, start=tf_start, end=tf_end).count(),
            'assembly3 should not have access anywhere just yet',
        )

        # create a share which is around the timeframe: should make room 1a assignable to assembly3
        share = self.room1a.shares.create(enabled=True, all_assemblies=True, timeframe=[tf_start - timedelta(minutes=1), tf_end + timedelta(minutes=1)])
        actual_qs = Room.objects.assignable_in_timeframe(self.conference, start=tf_start, end=tf_end, assembly=self.assembly3)
        self.assertSetEqual({'1a'}, set(actual_qs.values_list('name', flat=True)))
        share.delete()

        # test again with matching start, end or both
        share = self.room1a.shares.create(enabled=True, all_assemblies=True, timeframe=[tf_start, tf_end + timedelta(minutes=1)])
        actual_qs = Room.objects.assignable_in_timeframe(self.conference, start=tf_start, end=tf_end, assembly=self.assembly3)
        self.assertSetEqual({'1a'}, set(actual_qs.values_list('name', flat=True)))
        share.delete()

        share = self.room1a.shares.create(enabled=True, all_assemblies=True, timeframe=[tf_start - timedelta(minutes=1), tf_end])
        actual_qs = Room.objects.assignable_in_timeframe(self.conference, start=tf_start, end=tf_end, assembly=self.assembly3)
        self.assertSetEqual({'1a'}, set(actual_qs.values_list('name', flat=True)))
        share.delete()

        share = self.room1a.shares.create(enabled=True, all_assemblies=True, timeframe=[tf_start, tf_end])
        actual_qs = Room.objects.assignable_in_timeframe(self.conference, start=tf_start, end=tf_end, assembly=self.assembly3)
        self.assertSetEqual({'1a'}, set(actual_qs.values_list('name', flat=True)))
        share.delete()

        # setup interleaving ranges: assembly1+2 share a 1h window in which both are shared with assembly3 (and both have a 1h window on their own)
        share1 = self.room1a.shares.create(enabled=True, all_assemblies=True, timeframe=[tf_middle - timedelta(minutes=90), tf_middle + timedelta(minutes=30)])
        share2 = self.room2a.shares.create(enabled=True, all_assemblies=True, timeframe=[tf_middle - timedelta(minutes=30), tf_middle + timedelta(minutes=90)])
        actual_qs = Room.objects.assignable_in_timeframe(
            self.conference,
            start=tf_middle - timedelta(minutes=10),
            end=tf_middle + timedelta(minutes=10),
            assembly=self.assembly3,
        )
        self.assertSetEqual({'1a', '2a'}, set(actual_qs.values_list('name', flat=True)), 'Both assemblies should be available.')
        actual_qs = Room.objects.assignable_in_timeframe(
            self.conference,
            start=tf_middle - timedelta(minutes=80),
            end=tf_middle - timedelta(minutes=10),
            assembly=self.assembly3,
        )
        self.assertSetEqual({'1a'}, set(actual_qs.values_list('name', flat=True)), 'Only first room should be available.')
        actual_qs = Room.objects.assignable_in_timeframe(
            self.conference,
            start=tf_middle + timedelta(minutes=10),
            end=tf_middle + timedelta(minutes=50),
            assembly=self.assembly3,
        )
        self.assertSetEqual({'2a'}, set(actual_qs.values_list('name', flat=True)), 'Only second room should be available.')
        share1.delete()
        share2.delete()
