from django.contrib.auth.models import AnonymousUser
from django.test import TestCase

from core.models.assemblies import Assembly
from core.models.conference import Conference, ConferenceTrack
from core.models.events import Event
from core.models.pages import StaticPage
from core.models.users import PlatformUser
from core.search import search


class SearchTests(TestCase):
    fixtures = ['anhalter.json']

    def setUp(self):
        self.conference = Conference.objects.get(slug='vogc')
        self.user = AnonymousUser()
        page_user = PlatformUser(username='foo')
        page_user.save()

        self.cocktail_page = StaticPage(conference=self.conference, slug='cocktail', title='Pangalaktischer Donnergurgler')
        self.cocktail_page.save()
        self.cocktail_page_rev1 = self.cocktail_page.revisions.create(
            title=self.cocktail_page.title,
            body="""Der **Pangalaktische Donnergurgler** ist der angeblich stärkste Drink im Universum.
Die Wirkung eines Pangalaktischen Donnergurglers ist in etwa so, „als ob man mit einem Goldbarren,
der in Zitronenscheiben gehüllt ist, das Gehirn aus dem Kopf gedroschen bekommt“.""",
            author=page_user,
        )
        self.cocktail_page_rev1.save()
        self.cocktail_page_rev2 = self.cocktail_page.revisions.create(
            title=self.cocktail_page.title,
            body="""Der **Pangalaktische Donnergurgler** ist der angeblich stärkste Drink der Galaxis.
Die Wirkung eines Pangalaktischen Donnergurglers ist in etwa so, „als ob man mit einem Goldbarren,
der in Zitronenscheiben gehüllt ist, das Gehirn aus dem Kopf gedroschen bekommt“.""",
            author=page_user,
        )
        self.cocktail_page_rev2.save()
        self.cocktail_page_rev2.set_public()

    def test_search_single_keyword(self):
        # test track name and event description
        results = list(search(conference=self.conference, user=self.user, search_term='Universum'))
        results_count = len(results)
        self.assertGreaterEqual(results_count, 2)

        has_event, has_page, has_track = False, False, False
        for x in results:
            if isinstance(x, ConferenceTrack) and x.slug == 'spaceway':
                has_track = True
            if isinstance(x, Event) and str(x.id) == '35e10dfc-11c2-475b-b682-da1ddd291770':
                has_event = True
            if isinstance(x, StaticPage):
                has_page = True

        self.assertTrue(has_track)
        self.assertTrue(has_event)
        self.assertFalse(has_page)

        self.cocktail_page_rev1.set_public()
        results2 = list(search(conference=self.conference, user=self.user, search_term='Universum'))
        self.assertGreater(len(results2), results_count, 'Expected an additional search result.')
        self.assertTrue(any(isinstance(x, StaticPage) and x.id == self.cocktail_page.id for x in results2))

        # test Assembly description
        results2 = list(search(conference=self.conference, user=self.user, search_term='Planet'))
        self.assertEqual(len(results2), 1)

        self.assertTrue(any(isinstance(x, Assembly) and str(x.id) == '98b4c68c-1dfc-4e2a-84ae-546ef682fcf2' for x in results2))

        results = list(search(conference=self.conference, user=self.user, search_term='Universum"'))
        self.assertEqual(results, [])
