from typing import Any

from django.core.exceptions import ValidationError
from django.utils.deconstruct import deconstructible
from django.utils.translation import gettext_lazy as _


@deconstructible
class FileSizeValidator:
    """
    A validator class that will check a file size against the given limit

    Raises:
        ValidationError: Raises a validation error if the file is to large.
    """

    max_size: int = 0

    def __init__(self, max_size: int):
        """
        Returns a validator that can be used to check for file size

        Args:
            max_size (int): Maximum size of file to upload
        """
        self.max_size = max_size

    def __call__(self, file):
        if file.size > self.max_size * 1024 * 1024:
            raise ValidationError(
                _('Validation__error_file_size_MB_exceeded %(max_size)d'),
                code='max_file_size',
                params={'max_size': self.max_size},
            )
        else:
            return file

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.max_size == other.max_size


@deconstructible
class ImageDimensionValidator:
    """
    A validator class that will check a image dimension against the given limits

    Raises:
        ValidationError: Raises a validation error if the image does not match the given dimensions.
    """

    min_size: tuple[int | None, int | None]
    max_size: tuple[int | None, int | None]
    square: bool

    def __init__(self, *, min_size: tuple[int | None, int | None] = (None, None), max_size: tuple[int | None, int | None] = (None, None), square: bool = False):
        """Return a model field validator for ImageFields that raises an ValidationError
            if the image has incorrect dimensions

        Args:
            min_size (tuple[int  |  None, int  |  None], optional): Minimum dimensions in px. Defaults to (None, None).
            max_size (tuple[int  |  None, int  |  None], optional): Maximum dimensions in px. Defaults to (None, None).
            square (bool, optional): Image must be square? Defaults to False.
        """
        self.min_size = min_size
        self.max_size = max_size
        self.square = square

    def __call__(self, image) -> Any:
        errors = []
        if self.min_size[0] is not None and image.width < self.min_size[0]:
            errors.append(
                ValidationError(
                    _('Validation__error_image_width_low %(size)d'),
                    code='min_width',
                    params={'size': self.min_size[0]},
                )
            )
        if self.min_size[1] is not None and image.height < self.min_size[1]:
            errors.append(
                ValidationError(
                    _('Validation__error_image_height_low %(size)d'),
                    code='min_height',
                    params={'size': self.min_size[1]},
                )
            )
        if self.max_size[0] is not None and image.width > self.max_size[0]:
            errors.append(
                ValidationError(
                    _('Validation__error_image_width_high %(size)d'),
                    code='max_width',
                    params={'size': self.max_size[0]},
                )
            )
        if self.max_size[1] is not None and image.height > self.max_size[1]:
            errors.append(
                ValidationError(
                    _('Validation__error_image_height_high %(size)d'),
                    code='max_height',
                    params={'size': self.max_size[1]},
                )
            )
        if errors:
            errors.append(
                ValidationError(
                    _('Validation__image_dimensions %(min_width)s %(min_height)s %(max_width)s %(max_height)s'),
                    code='image_dimensions',
                    params={
                        'min_width': self.min_size[0],
                        'min_height': self.min_size[1],
                        'max_width': self.max_size[0],
                        'max_height': self.max_size[1],
                    },
                )
            )

        if self.square and image.width != image.height:
            errors.append(ValidationError(_('Validation__error_image_square')))
        if errors:
            raise ValidationError(errors)
        else:
            return image

    def __eq__(self, other):
        return isinstance(other, self.__class__) and (
            self.min_size,
            self.max_size,
            self.square,
        ) == (
            other.min_size,
            other.max_size,
            other.square,
        )


def validate_lower_case(value: str) -> None:
    """
    A validator function that will check if a string is lowercase

    Raises:
        ValidationError: Raises a validation error if the string is not lowercase.

    Args:
        value (str): The string to check
    """
    if value != value.lower():
        raise ValidationError(_('Validation__error_lowercase'), code='lowercase')
