import json

from django_ratelimit.exceptions import Ratelimited

from django.conf import settings
from django.contrib.auth import get_user_model
from django.http import HttpResponse, HttpResponseForbidden, HttpResponseServerError, JsonResponse
from django.utils.cache import add_never_cache_headers
from django.views.decorators.http import require_safe
from django.views.i18n import set_language as set_language_internal

from hub.http import HttpResponseRateLimited


@require_safe
def version(request):
    resp = JsonResponse(settings.APP_VERSION_INFO)
    add_never_cache_headers(resp)
    return resp


@require_safe
def health(request):
    issues = []

    User = get_user_model()
    if User.objects.count() == 0:
        issues.append('No users.')

    # TODO: check other apps for issues

    if len(issues) == 0:
        response = HttpResponse('OK', 'text/plain')
    else:
        response = HttpResponse('\n'.join(issues), 'text/plain', status=418)

    add_never_cache_headers(response)
    return response


@require_safe
def index(request):
    return HttpResponseServerError(content=b'Server Configuration Error - no API, no BACKOFFICE and no FRONTEND :(')


def debug_headers(request):
    param = request.GET if request.method not in ['POST', 'DELETE', 'PUT', 'PATCH'] else request.POST
    token = param.get('token', '')
    if token != settings.HEADERS_DEBUG_ENDPOINT_TOKEN:
        return HttpResponse('Wrong token.', 'text/plain', status=403)

    request_headers = dict(request.headers)
    return HttpResponse(json.dumps(request_headers, indent=2), 'application/json', status=200)


def handle403(request, exception=None):
    if isinstance(exception, Ratelimited):
        return HttpResponseRateLimited('Rate limit exceeded')
    return HttpResponseForbidden('Forbidden')


def set_language(request):
    """
    Strip the SCRIPT_NAME from the next URL before passing it to the
    set_language view.
    """
    next_url = request.POST.get('next', request.GET.get('next'))
    if next_url:
        if next_url.startswith(settings.SCRIPT_NAME):
            next_url = next_url[len(settings.SCRIPT_NAME) :]
        request.POST = request.POST.copy()
        request.POST['next'] = next_url
    return set_language_internal(request)
